@@ -5,15 +5,29 @@ import datetime
 import decimal
 import logging
 import os
-import Queue
 import threading
 import unittest
+import sys
+try:
+    from Queue import Queue
+except ImportError:
+    from queue import Queue
 
 from peewee import *
-from peewee import QueryCompiler, R, SelectQuery, RawQuery, InsertQuery,\
-    UpdateQuery, DeleteQuery, logger, transaction, sort_models_topologically,\
-    prefetch_add_subquery, NaiveQueryResultWrapper, ModelQueryResultWrapper
-
+from peewee import DeleteQuery
+from peewee import InsertQuery
+from peewee import logger
+from peewee import ModelQueryResultWrapper
+from peewee import NaiveQueryResultWrapper
+from peewee import prefetch_add_subquery
+from peewee import print_
+from peewee import QueryCompiler
+from peewee import R
+from peewee import RawQuery
+from peewee import SelectQuery
+from peewee import sort_models_topologically
+from peewee import transaction
+from peewee import UpdateQuery
 
 
 class QueryLogHandler(logging.Handler):
@@ -24,6 +38,12 @@ class QueryLogHandler(logging.Handler):
     def emit(self, record):
         self.queries.append(record)
 
+if sys.version_info[0] < 3:
+    import codecs
+    ulit = lambda s: codecs.unicode_escape_decode(s)[0]
+else:
+    ulit = lambda s: s
+
 #
 # JUNK TO ALLOW TESTING OF MULTIPLE DATABASE BACKENDS
 #
@@ -33,6 +53,8 @@ TEST_VERBOSITY = int(os.environ.get('PEEWEE_TEST_VERBOSITY') or 1)
 
 database_params = {}
 
+print_('TESTING USING PYTHON %s' % sys.version)
+
 if BACKEND == 'postgresql':
     database_class = PostgresqlDatabase
     database_name = 'peewee_test'
@@ -48,7 +70,7 @@ else:
     database_class = SqliteDatabase
     database_name = 'tmp.db'
     import sqlite3
-    print 'SQLITE VERSION: %s' % sqlite3.version
+    print_('SQLITE VERSION: %s' % sqlite3.version)
 
 #
 # TEST-ONLY QUERY COMPILER USED TO CREATE "predictable" QUERIES
@@ -657,7 +679,7 @@ class SelectTestCase(BasePeeweeTestCase):
         fixed_sql = [
             ('SELECT t1."id", t1."username" FROM "users" AS t1 WHERE (t1."username" = ?)', ['foo']),
             ('SELECT t1."pk", t1."user_id", t1."title", t1."content", t1."pub_date" FROM "blog" AS t1 WHERE ((t1."title" = ?) AND (t1."user_id" IN (SELECT t2."id" FROM "users" AS t2 WHERE (t2."username" = ?))))', ['bar', 'foo']),
-            ('SELECT t1."id", t1."blog_id", t1."comment" FROM "comment" AS t1 WHERE ((t1."comment" = ?) AND (t1."blog_id" IN (SELECT t2."pk" FROM "blog" AS t2 WHERE ((t2."title" = ?) AND (t2."user_id" IN (SELECT t3."id" FROM "users" AS t3 WHERE (t3."username" = ?)))))))', [u'baz', u'bar', u'foo']),
+            ('SELECT t1."id", t1."blog_id", t1."comment" FROM "comment" AS t1 WHERE ((t1."comment" = ?) AND (t1."blog_id" IN (SELECT t2."pk" FROM "blog" AS t2 WHERE ((t2."title" = ?) AND (t2."user_id" IN (SELECT t3."id" FROM "users" AS t3 WHERE (t3."username" = ?)))))))', ['baz', 'bar', 'foo']),
         ]
         for (query, fkf), expected in zip(fixed, fixed_sql):
             self.assertEqual(normal_compiler.generate_select(query), expected)
@@ -1187,6 +1209,23 @@ class ModelQueryTestCase(ModelTestCase):
             ('u1',), ('u2',), ('u3',),
         ])
 
+    def test_limits_offsets(self):
+        for i in range(10):
+            self.create_user(username='u%d' % i)
+        sq = User.select().order_by(User.id)
+
+        offset_no_lim = sq.offset(3)
+        self.assertEqual(
+            [u.username for u in offset_no_lim],
+            ['u%d' % i for i in range(3, 10)]
+        )
+
+        offset_with_lim = sq.offset(5).limit(3)
+        self.assertEqual(
+            [u.username for u in offset_with_lim],
+            ['u%d' % i for i in range(5, 8)]
+        )
+
 
 class ModelAPITestCase(ModelTestCase):
     requires = [User, Blog, Category, UserCategory]
@@ -1441,7 +1480,7 @@ class ModelAPITestCase(ModelTestCase):
 
     def test_unicode(self):
         # create a unicode literal
-        ustr = u'Lýðveldið Ísland'
+        ustr = ulit('Lýðveldið Ísland')
         u = self.create_user(username=ustr)
 
         # query using the unicode literal
@@ -2171,7 +2210,7 @@ class ConcurrencyTestCase(ModelTestCase):
         self.assertEqual(User.select().count(), 50)
 
     def test_multiple_readers(self):
-        data_queue = Queue.Queue()
+        data_queue = Queue()
 
         def reader_thread(q, num):
             for i in range(num):
@@ -2379,7 +2418,7 @@ def permutations(xs):
                 yield [y] + pys
 
 def selections(xs):
-    for i in xrange(len(xs)):
+    for i in range(len(xs)):
         yield (xs[i], xs[:i] + xs[i + 1:])
 
 
@@ -2419,7 +2458,7 @@ if test_db.for_update:
             self.assertEqual(username, 'u1_edited')
 
 elif TEST_VERBOSITY > 0:
-    print 'Skipping "for update" tests'
+    print_('Skipping "for update" tests')
 
 if test_db.sequences:
     class SequenceTestCase(ModelTestCase):
@@ -2438,7 +2477,7 @@ if test_db.sequences:
             self.assertEqual(b2.id, a3.id - 1)
 
 elif TEST_VERBOSITY > 0:
-    print 'Skipping "sequence" tests'
+    print_('Skipping "sequence" tests')
 
 
 class Job(TestModel):
